#################################################################################
#                                                                               
#                       copyright (c) 2003 Ingeniweb SARL                       
#                       copyright (c) 2006 Felix Kurth                                                          
#################################################################################

"""I18N Folder main class"""

from AccessControl import ClassSecurityInfo
import string

from Products.CMFCore.permissions import View,ModifyPortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.Archetypes.public import *
from Products.ATContentTypes.content.folder import ATFolder as Base
from Products.ATContentTypes.content.document import finalizeATCTSchema
from Products.ATContentTypes.content.schemata import ATContentTypeSchema

from zExceptions import Redirect 

from Products.LinguaPlone.public import AlreadyTranslated
from Products.CMFPlone.interfaces.Translatable import ITranslatable
from Products.I18NFolder.config import *
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_base,aq_inner,aq_parent
from Products.CMFPlone.utils import safe_callable
from Products.Archetypes.utils import shasattr
from Products.ATContentTypes.content.folder import ATFolderSchema 


schema = ATFolderSchema.copy()
finalizeATCTSchema(schema, folderish=True)


class I18NFolder (Base):
    """Translatable Folder
    """
    
    __implements__ = Base.__implements__ 
    
    portal_type = meta_type = 'I18NFolder'
    archetype_name = 'IFolder'
    content_icon = 'i18nfolder_icon.gif'
    #schema = schema
    
    typeDescription= 'A multilingual folder'
    typeDescMsgId  = 'I18NFolder_description_edit'

    default_view = 'folder_listing'
    immediate_view = 'folder_listing'
    #suppl_views = ('ifwdocument_view_preview', 'ifwdocument_view_float', 'folder_listing', 'folder_summary_view', 'folder_tabular_view', 'atct_album_view')

    _at_rename_after_creation = True
    LanguageFixed = None
    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        # NOTA : We shouldn't call parent's __init__ method as it would link to PortalFolder.__init__ and this
        # method sets 'self.id' and 'self.title' which is unuseful for us.
        #self.id = id
        self.folder_languages = {}
        self.LanguageFixed = None
        Base.__init__(self,id)
    
    
    # getCurrentLanguage method
    security.declareProtected(View, 'getCurrentLanguage')
    def getCurrentLanguage(self):
        """
        getCurrentLanguage(self)
        """
        languageTool = getToolByName(self, 'portal_languages', None)
        if languageTool:
            current_language = languageTool.getPreferredLanguage()
        else:
            current_language = 'en'
        return current_language
        
    # getLanguageObject method. Use fallback
    security.declarePrivate('getLanguageObject')
    def getLanguageObject(self, language=None, fallback=True):
        """
        getLanguageObject(self)
        """
        if language is None:
            language = self.getCurrentLanguage()
        #while reindexing....
        if self.LanguageFixed:
            language = self.LanguageFixed
        
        if self.hasTranslation(language):
            return self.folder_languages[language]
        if self.hasTranslation('neutral'):
            return self.folder_languages['neutral']
        if not fallback:
            return None
        language = self.getDefaultLanguage()
        if self.hasTranslation(language):
            return self.folder_languages[language]
        l = self.folder_languages.keys()
        return self.folder_languages[l[0]]
        
    #Overwrite this method, in your dereived classes to use more fields...
    security.declarePrivate('createLanguageObject')
    def createLanguageObject(self,lang='neutral'):
        language_object = {'title' : '', 'description' : '','cid' : '', 'lang' : lang}
        return language_object 
    
    # Set title method
    security.declareProtected(ModifyPortalContent, 'setTitle')
    def setTitle(self, title='', language=None):
        """
        setTitle(self, title='', lang_code=None)
        """
        language_object = self.getLanguageObject(language)
        language = language_object['lang']
        language_object['title'] = title
        self.folder_languages[language] = language_object
            
    # Title method
    security.declareProtected(View, 'Title')
    def Title(self, lang_code=None):
        """
        getTitle(self)
        """
        # Can't return a title if there are no languages in the folder    
        if self.folder_languages:
            language_object = self.getLanguageObject(lang_code)
            title = language_object['title']
        else:
            title = ""        
        return title
    
    # title_or_id method
    security.declareProtected(View, 'title_or_id')
    def title_or_id(self, lang_code=None):
        """
        title_or_id(self)
        """
        title = self.Title(lang_code)
        
        if title:
            return title
        else:
            return self.getId()
    
    
    # Set description method
    security.declareProtected(ModifyPortalContent, 'setDescription')
    def setDescription(self, description='', language=None):
        """
        setDescription(self, description='', lang_code=None)
        """
        language_object = self.getLanguageObject(language)
        language = language_object['lang']
        language_object['description'] = description
        self.folder_languages[language] = language_object
        
    # Description method
    security.declareProtected(View, 'Description')
    def Description(self, lang_code=None):
        """
        getDescription(self)
        """
        language_object = self.getLanguageObject(lang_code)
        description = language_object['description']
        return description
    
    security.declareProtected(View, 'getDescription')
    def getDescription(self, lang_code=None):
        return self.Description(lang_code)
   
    security.declareProtected(View, 'getRawDescription')
    def getRawDescription(self, lang_code=None):
        return self.Description(lang_code)
   
    security.declareProtected(View, 'isTranslation')
    def isTranslation(self):
        o = self.getLanguageObject()
        if not o:
            return False
        if o['lang'] == 'neutral':
            return False
        return True
    
    # getLanguages method. Return a dictionnary of available languages
    security.declareProtected(View, 'getLanguages')
    def getLanguages(self):
        """
        getLanguages(self)
        """
        languages_list = []
        languages = {}
        
        # Test if portal_languages tool exists
        try:
            languages_list = self.portal_languages.listSupportedLanguages()
        except:
            # So use default languages list
            languages_list = self.availableLanguages()
        
        # Build dictionary    
        for language in languages_list:
            languages[language[0]] = language[1]
        return languages
        
    # getDefaultLanguage method
    security.declareProtected(View, 'getDefaultLanguage')
    def getDefaultLanguage(self):
        """
        getDefaultLanguage(self)
        """
        languageTool = getToolByName(self, 'portal_languages', None)
        if languageTool:
            return languageTool.getDefaultLanguage()
        else:
            return 'en'
        
    # getDefinedLanguages method
    security.declareProtected(View, 'getDefinedLanguages')
    def getDefinedLanguages(self):
        """
        getDefinedLanguages(self)
        Languages wich are already translated-
        """
        return self.getTranslations().keys()
    
        
    # getUndefinedLanguages method
    security.declareProtected(View, 'getUndefinedLanguages')
    def getUndefinedLanguages(self):
        """
        getUndefinedLanguages(self)
        """
        
        defined_languages = self.folder_languages.keys()
        undefined_languages = self.getLanguages()
        
        # Delete defined languages from undefined_languages
        for code in defined_languages:
            del(undefined_languages[code])
        return undefined_languages
    
    
    # Add all languages in catalog
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        search = []
        language_object = self.getLanguageObject()
        
        if language_object:
            title = language_object['title']
            description = language_object['description']
            if title:
                search.append(title)
            if description:
                search.append(description)
        ret = string.join(search, ' ')
        return ret
    
    security.declareProtected(View, 'getTranslationLanguages')
    def getTranslationLanguages(self):
        """
        Return a list of language codes.
        """
        return self.getTranslations().keys()        

    security.declareProtected(View, 'getNonCanonicalTranslations')
    def getNonCanonicalTranslations(self):
        canonical = self.getDefaultLanguage()
        all = self.getTranslations()
        if '' in all:
            all.pop('')
        if canonical in all:
            all.pop(canonical)
                
        return all
    
    security.declareProtected(View, 'getTranslations')
    def getTranslations(self):
        """
        Return a dict of {lang : [object, wf_state]}
        """
        result = {}
        workflow_tool = getToolByName(self, 'portal_workflow', None)
        state = workflow_tool.getInfoFor(self, 'review_state', None)
        for lang in self.folder_languages:
            if not lang == 'neutral':
                result[lang] = [self, state]
            else:
                result[''] = [self, state]
        return result
    
    security.declareProtected(ModifyPortalContent, 'removeTranslation')
    def removeTranslation(self,lang):
        if self.hasTranslation(lang):
            #The last on..., go to neutral, but dont delete...
            if len(self.folder_languages) == 1:
                self.setLanguage('neutral')
                return
            self.unindexObject(lang)
            del(self.folder_languages[lang])
        
    security.declareProtected(ModifyPortalContent, 'invalidateTranslations')
    def invalidateTranslations(self):
        """Outdates all translations except the canonical one."""
        # only fake here
        pass

    def isOutdated(self):
        """Checks if the translation is outdated."""
        # fake here
        return False

    security.declareProtected(View, 'isCanonical')
    def isCanonical(self):
        """Returns the canonical translation."""
        return True
    
    
    security.declareProtected(View, 'getCanonical')
    def getCanonical(self):
        """Returns the canonical translation."""
        return self
    
    security.declareProtected(View, 'getCanonicalLanguage')
    def getCanonicalLanguage(self):
        """Returns the canonical translation."""
        lo = self.getLanguageObject(self.getDefaultLanguage)
        if lo and lo['lang'] == 'neutral':
            return None
        if lo:
            return lo['lang']
        return None
        
    
    security.declareProtected(View, 'getTranslation')
    def getTranslation(self, language=None):
        if language == None:
            return self
        if self.hasTranslation(language):
            return self
        return None
        

    security.declareProtected(ModifyPortalContent, 'addTranslation')
    def addTranslation(self, language, *args, **kwargs):
        """Adds a translation."""
        if self.hasTranslation(language):
            raise AlreadyTranslated
        if language == 'neutral':
            return
        if self.hasTranslation('neutral'):
            self.setLanguage(language,'neutral')
        else:   
            self.folder_languages[language] = self.createLanguageObject(language)
            self.reindexObject()
        
    security.declareProtected(View, 'hasTranslation')
    def hasTranslation(self,language):
        return self.folder_languages and self.folder_languages.has_key(language)
        
        
    security.declareProtected(View, 'getLanguage')
    def getLanguage(self):
        """Returns the language code."""
        return self.Language()
    
    security.declareProtected(ModifyPortalContent, 'setLanguage')
    def setLanguage(self,language,current=None):
        """
        Sets the language for the current translation 
        """
        if language=='':
            language = 'neutral'
        if current is None:
            currentobj = self.getLanguageObject(current,False)
            if not currentobj:
                return
            current = currentobj['lang']
        #we are called with an explicit current lang, but its not here.    
        if not self.hasTranslation(current):
            return
        if current== language:
            return
        if self.hasTranslation(language):
            raise AlreadyTranslated, self.absolute_url()
        
        new_lang_object = currentobj.copy()
        new_lang_object['lang'] = language
        self.folder_languages[language] = new_lang_object
        self.removeTranslation(current)
        #clean completely up if we change to neutral
        if language=='neutral':
            trans = self.getTranslationLanguages()
            for t in trans:
                self.removeTranslation(t)
        self.reindexObject()
        return

    security.declareProtected(View, 'Language')
    def Language(self):
        #The catalog ask us while reindexing... 
        if self.LanguageFixed:
            if self.LanguageFixed=='neutral':
                return ''
            return self.LanguageFixed
        o = self.getLanguageObject()
        if o['lang'] == 'neutral':
            return ''
        return o['lang']
    
    
    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        """
            Index the object in the portal catalog.
        """
        self.reindexObject()
        
    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self, lang=None):
        """
            Unindex the object from the portal catalog.
        """
        catalog = self._getCatalogTool()
        if catalog is not None:
            if lang:
                uid = self.getCid(lang)
                if uid:
                    catalog.uncatalog_object(uid,True)
            else:
                langs = self.folder_languages.keys()
                for lang in langs:
                    uid = self.getCid(lang)
                    if uid:
                        catalog.uncatalog_object(uid,True)

    def getCid(self,lang):
        uid = '/'.join(self.getPhysicalPath())
        if lang == 'neutral':
            return uid
        return uid + '--i18nfolder-' + lang
        
    
    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[], update_metadata=1):
        """
            Reindex the object in the portal catalog.
            If idxs is present, only those indexes are reindexed.
            The metadata is always updated.
            
            Also update the modification date of the object,
            unless specific indexes were requested.
            
            We create an entry for every language!! This is possible
            because the AbstractCatalogBrain,getPath() Method
            is patched. 
            See patches.py
        """
        if idxs == [] and shasattr(self, 'notifyModified'):
            # Archetypes default setup has this defined in ExtensibleMetadata
            # mixin. note: this refreshes the 'etag ' too.
            self.notifyModified()

        self.http__refreshEtag()

        catalog = self._getCatalogTool()
        if catalog is not None:
      
            #we clean completley up
            #this is neccessary because in the case of move-actions
            if idxs == [] :
                self.unindexObject()
            #catalog.uncatalog_object(uid)
            
            for lang in self.folder_languages:
                self.LanguageFixed = lang
                id = self.getCid(lang)
                #catalog.catalog_object(self, idxs, update_metadata, id, None, True)
                catalog.catalog_object(self, id, idxs, update_metadata, None, True)
                #self.folder_languages[lang]['cid'] = id
            self.LanguageFixed  = None
            
            
    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        Base.manage_afterClone(self, item)
        self.reindexObject()
    
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """
            Add self to the catalog.
            (Called when the object is created or moved.)
        """
        if not self.folder_languages:
            self.folder_languages['neutral'] = self.createLanguageObject('neutral')
        self.indexObject()
        Base.manage_afterAdd(self, item, container)
        
    
registerType(I18NFolder)

